from qtpy.QtGui import QDoubleValidator, QRegExpValidator, QValidator
from qtpy.QtCore import QRegExp
from qtdraw.core.pixmap_converter import _latex2pixmap


"""
regular-expression patterns.
    - see, https://regexlib.com/CheatSheet.aspx
"""
# scalar.
int_ptn = "[+-]?[0-9]+"
float_ptn = "[+-]?([0-9]+([.][0-9]*)?|[.][0-9]+)"
str_ptn = "[^\[\],\"']*"  # except [],"
int_pos_ptn = "[+]?[0-9]+"
float_pos_ptn = "[+]?([0-9]+([.][0-9]*)?|[.][0-9]+)"

# 3-component vector.
int_vector_ptn = "^\[(" + int_ptn + ",){2}" + int_ptn + "\]$"
int_pos_vector_ptn = "^\[(" + int_pos_ptn + ",){2}" + int_pos_ptn + "\]$"
float_vector_ptn = "^\[(" + float_ptn + ",){2}" + float_ptn + "\]$"
float_pos_vector_ptn = "^\[(" + float_pos_ptn + ",){2}" + float_pos_ptn + "\]$"
str_vector_ptn = "^\[(" + str_ptn + ",){2}" + str_ptn + "\]$"

# list.
int_list_ptn = "^\[(" + int_ptn + ",)*" + int_ptn + "\]$"
int_pos_list_ptn = "^\[(" + int_pos_ptn + ",)*" + int_pos_ptn + "\]$"
float_list_ptn = "^\[(" + float_ptn + ",)*" + float_ptn + "\]$"
float_pos_list_ptn = "^\[(" + float_pos_ptn + ",)*" + float_pos_ptn + "\]$"
str_list_ptn = "^\[(" + str_ptn + ",)*" + str_ptn + "\]$"

# list of 3-component vector.
int_vector_list_ptn = "^\[(" + int_vector_ptn[1:-1] + ",)*" + int_vector_ptn[1:-1] + "\]$"
int_pos_vector_list_ptn = "^\[(" + int_pos_vector_ptn[1:-1] + ",)*" + int_pos_vector_ptn[1:-1] + "\]$"
float_vector_list_ptn = "^\[(" + float_vector_ptn[1:-1] + ",)*" + float_vector_ptn[1:-1] + "\]$"
float_pos_vector_list_ptn = "^\[(" + float_pos_vector_ptn[1:-1] + ",)*" + float_pos_vector_ptn[1:-1] + "\]$"
str_vector_list_ptn = "^\[(" + str_vector_ptn[1:-1] + ",)*" + str_vector_ptn[1:-1] + "\]$"

# list of list.
int_list_list_ptn = "^\[(" + int_list_ptn[1:-1] + ",)*" + int_list_ptn[1:-1] + "\]$"
int_pos_list_list_ptn = "^\[(" + int_pos_list_ptn[1:-1] + ",)*" + int_pos_list_ptn[1:-1] + "\]$"
float_list_list_ptn = "^\[(" + float_list_ptn[1:-1] + ",)*" + float_list_ptn[1:-1] + "\]$"
float_pos_list_list_ptn = "^\[(" + float_pos_list_ptn[1:-1] + ",)*" + float_pos_list_ptn[1:-1] + "\]$"
str_list_list_ptn = "^\[(" + str_list_ptn[1:-1] + ",)*" + str_list_ptn[1:-1] + "\]$"


class UnitValidator(QDoubleValidator):
    def __init__(self, parent=None):
        super().__init__(0.0, 1.0, 0, parent)
        self.setNotation(QDoubleValidator.StandardNotation)


class LaTeXValidator(QValidator):
    def __init__(self, parent=None):
        super().__init__(parent)

    def validate(self, text, pos):
        if text.count("$"):
            text = text.strip("$")
            flag = _latex2pixmap(text)
            if flag is None:
                return QValidator.Invalid, text, pos
            else:
                text = "$" + text + "$"
        return QValidator.Acceptable, text, pos


"""
validator mapping.
"""
validator_map = {
    "int": int_ptn,
    "real": float_ptn,
    "string": str_ptn,
    "int_positive": int_pos_ptn,
    "real_positive": float_pos_ptn,
    "real_unit": "real_unit",
    "latex": "latex",
    #
    "i_scalar": int_ptn,
    "i_pos_scalar": int_pos_ptn,
    "i_row": int_list_ptn,
    "i_pos_row": int_pos_list_ptn,
    "i_column": int_list_ptn,
    "i_pos_column": int_pos_list_ptn,
    "i_matrix": int_list_list_ptn,
    "i_pos_matrix": int_pos_list_list_ptn,
    "i_vector": int_vector_ptn,
    "i_pos_vector": int_pos_vector_ptn,
    "i_column_vector": int_vector_ptn,
    "i_pos_column_vector": int_pos_vector_ptn,
    "i_vector_list": int_vector_list_ptn,
    "i_pos_vector_list": int_pos_vector_list_ptn,
    #
    "r_scalar": float_ptn,
    "r_pos_scalar": float_pos_ptn,
    "r_row": float_list_ptn,
    "r_pos_row": float_pos_list_ptn,
    "r_column": float_list_ptn,
    "r_pos_column": float_pos_list_ptn,
    "r_matrix": float_list_list_ptn,
    "r_pos_matrix": float_pos_list_list_ptn,
    "r_vector": float_vector_ptn,
    "r_pos_vector": float_pos_vector_ptn,
    "r_column_vector": float_vector_ptn,
    "r_pos_column_vector": float_pos_vector_ptn,
    "r_vector_list": float_vector_list_ptn,
    "r_pos_vector_list": float_pos_vector_list_ptn,
    #
    "s_scalar": str_ptn,
    "s_row": str_list_ptn,
    "s_column": str_list_ptn,
    "s_matrix": str_list_list_ptn,
    "s_vector": str_vector_ptn,
    "s_column_vector": str_vector_ptn,
    "s_vector_list": str_vector_list_ptn,
}


def create_validator(validator):
    """
    create validator.

    Args:
        validator (str): name of validator. see, validator_map.keys()

    Returns:
        QValidator: validator.
    """
    if validator == "real_unit":
        return UnitValidator()
    elif validator == "latex":
        return LaTeXValidator()
    else:
        return QRegExpValidator(QRegExp(validator_map[validator]))



from qtpy.QtWidgets import QWidget, QTreeView, QGridLayout, QHeaderView, QInputDialog, QPushButton
from qtpy.QtCore import Qt, Signal
from qtdraw.core.group_model import GroupModel, Delegate
from qtdraw.core.setting import rcParams


# ==================================================
class TreeView(QTreeView):
    deselectedItem = Signal()

    def mousePressEvent(self, event):
        self.deselectedItem.emit()
        super().mousePressEvent(event)

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Down or event.key() == Qt.Key_Up:
            self.deselectedItem.emit()
            super().keyPressEvent(event)
            self.clicked.emit(self.selectionModel().currentIndex())
        else:
            super().keyPressEvent(event)


# ==================================================
class GroupView(QWidget):
    textMessage = Signal(str)
    selectedData = Signal(str, int)
    deselectedData = Signal()

    # ==================================================
    def __init__(self, ds, name, read_only=False, align=None, parent=None):
        super().__init__(parent)

        self.name = name
        self.model = GroupModel(ds, name, read_only, align, parent=self)

        self.view = TreeView(self)
        self.view.setModel(self.model)
        bg = rcParams["detail.table.bg_color"]
        self.view.setStyleSheet(f"selection-background-color: {bg}")
        self.view.clicked.connect(self.itemSelected)
        self.view.deselectedItem.connect(self.itemDeselected)

        delegate = Delegate(bg, self.view)
        self.view.setItemDelegate(delegate)

        for column in self.model.hide:
            self.view.header().setSectionHidden(column, True)

        for column in range(self.model.columnCount()):
            self.view.header().setSectionResizeMode(column, QHeaderView.ResizeToContents)
            self.view.header().setDefaultAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        self.view.setAlternatingRowColors(True)
        self.view.header().setSectionsMovable(False)
        self.view.header().setStretchLastSection(False)

        if not read_only:
            self.add_button = QPushButton("Add", self)
            self.insert_button = QPushButton("Insert", self)
            self.remove_button = QPushButton("Remove", self)
            self.add_button.setFocusPolicy(Qt.NoFocus)
            self.insert_button.setFocusPolicy(Qt.NoFocus)
            self.remove_button.setFocusPolicy(Qt.NoFocus)

        layout = QGridLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)

        if not read_only:
            layout.addWidget(self.view, 0, 0, 1, 3)
            layout.addWidget(self.add_button, 1, 0)
            layout.addWidget(self.insert_button, 1, 1)
            layout.addWidget(self.remove_button, 1, 2)

            self.add_button.clicked.connect(self.add)
            self.insert_button.clicked.connect(self.insert)
            self.remove_button.clicked.connect(self.remove)
        else:
            layout.addWidget(self.view, 0, 0)

        self.setLayout(layout)

    # ==================================================
    def itemSelected(self, index):
        indices = index.model().childIndex(index)
        if len(indices) == 1:  # no child.
            row = index.model().absIndex(indices[0])[0]
            self.selectedData.emit(self.name, row)
        else:  # with child.
            for idx in indices[:-1]:
                row = index.model().absIndex(idx)[0]
                self.selectedData.emit(self.name, row)

    # ==================================================
    def itemDeselected(self):
        self.view.clearSelection()
        self.deselectedData.emit()

    # ==================================================
    def add(self):
        text, ok = QInputDialog.getText(self, self.model.name, "Name:")
        if ok:
            self.itemDeselected()
            self.model.appendRow(name=str(text))
            self.textMessage.emit(f"add '{text}' object into {self.name}.")

    # ==================================================
    def insert(self):
        index = self.view.selectedIndexes()
        if len(index) != 0:
            self.itemDeselected()
            self.model.appendRow(index[0])
            self.textMessage.emit(f"insert object into {self.name}.")

    # ==================================================
    def remove(self):
        index = self.view.selectedIndexes()
        if len(index) != 0:
            self.itemDeselected()
            self.model.removeRow(index[0])
            self.textMessage.emit(f"remove object from {self.name}.")
